public inbox for pbs-devel@lists.proxmox.com
 help / color / mirror / Atom feed
From: Christian Ebner <c.ebner@proxmox.com>
To: pbs-devel@lists.proxmox.com
Subject: [PATCH proxmox-backup v4 25/30] sync: pull: extend encountered chunk by optional decrypted digest
Date: Mon, 20 Apr 2026 18:15:28 +0200	[thread overview]
Message-ID: <20260420161533.1055484-26-c.ebner@proxmox.com> (raw)
In-Reply-To: <20260420161533.1055484-1-c.ebner@proxmox.com>

For index files being decrypted during the pull, it is not enough to
keep track of the processes source chunks, but the decrypted digest
has to be known as well in order to rewrite the index file.

Extend the encountered chunks such that this can be tracked as well.
To not introduce clippy warnings and to keep the code readable,
introduce the EncounteredChunksInfo struct as internal type for the
hash map values.

Signed-off-by: Christian Ebner <c.ebner@proxmox.com>
---
 src/server/pull.rs | 78 +++++++++++++++++++++++++++++++++-------------
 1 file changed, 56 insertions(+), 22 deletions(-)

diff --git a/src/server/pull.rs b/src/server/pull.rs
index 174e50a91..1c1c9458e 100644
--- a/src/server/pull.rs
+++ b/src/server/pull.rs
@@ -176,7 +176,7 @@ async fn pull_index_chunks<I: IndexFile>(
             .filter(|info| {
                 let guard = encountered_chunks.lock().unwrap();
                 match guard.check_reusable(&info.digest) {
-                    Some(touched) => !touched, // reusable and already touched, can always skip
+                    Some(reusable) => !reusable.touched, // reusable and already touched, can always skip
                     None => true,
                 }
             }),
@@ -212,22 +212,22 @@ async fn pull_index_chunks<I: IndexFile>(
             {
                 // limit guard scope
                 let mut guard = encountered_chunks.lock().unwrap();
-                if let Some(touched) = guard.check_reusable(&info.digest) {
-                    if touched {
+                if let Some(reusable) = guard.check_reusable(&info.digest) {
+                    if reusable.touched {
                         return Ok::<_, Error>(());
                     }
                     let chunk_exists = proxmox_async::runtime::block_in_place(|| {
                         target.cond_touch_chunk(&info.digest, false)
                     })?;
                     if chunk_exists {
-                        guard.mark_touched(&info.digest);
+                        guard.mark_touched(&info.digest, None);
                         //info!("chunk {} exists {}", pos, hex::encode(digest));
                         return Ok::<_, Error>(());
                     }
                 }
                 // mark before actually downloading the chunk, so this happens only once
-                guard.mark_reusable(&info.digest);
-                guard.mark_touched(&info.digest);
+                guard.mark_reusable(&info.digest, None);
+                guard.mark_touched(&info.digest, None);
             }
 
             //info!("sync {} chunk {}", pos, hex::encode(digest));
@@ -828,7 +828,7 @@ async fn pull_group(
 
                         for pos in 0..index.index_count() {
                             let chunk_info = index.chunk_info(pos).unwrap();
-                            reusable_chunks.mark_reusable(&chunk_info.digest);
+                            reusable_chunks.mark_reusable(&chunk_info.digest, None);
                         }
                     }
                 }
@@ -1266,12 +1266,23 @@ async fn pull_ns(
     Ok((progress, sync_stats, errors))
 }
 
+struct EncounteredChunkInfo {
+    reusable: bool,
+    touched: bool,
+    decrypted_digest: Option<[u8; 32]>,
+}
+
 /// Store the state of encountered chunks, tracking if they can be reused for the
 /// index file currently being pulled and if the chunk has already been touched
 /// during this sync.
 struct EncounteredChunks {
-    // key: digest, value: (reusable, touched)
-    chunk_set: HashMap<[u8; 32], (bool, bool)>,
+    chunk_set: HashMap<[u8; 32], EncounteredChunkInfo>,
+}
+
+/// Propertires of a reusable chunk
+struct ReusableEncounteredChunk<'a> {
+    touched: bool,
+    decrypted_digest: Option<&'a [u8; 32]>,
 }
 
 impl EncounteredChunks {
@@ -1284,41 +1295,64 @@ impl EncounteredChunks {
 
     /// Check if the current state allows to reuse this chunk and if so,
     /// if the chunk has already been touched.
-    fn check_reusable(&self, digest: &[u8; 32]) -> Option<bool> {
-        if let Some((reusable, touched)) = self.chunk_set.get(digest) {
-            if !reusable {
+    fn check_reusable(&self, digest: &[u8; 32]) -> Option<ReusableEncounteredChunk<'_>> {
+        if let Some(chunk_info) = self.chunk_set.get(digest) {
+            if !chunk_info.reusable {
                 None
             } else {
-                Some(*touched)
+                Some(ReusableEncounteredChunk {
+                    touched: chunk_info.touched,
+                    decrypted_digest: chunk_info.decrypted_digest.as_ref(),
+                })
             }
         } else {
             None
         }
     }
 
-    /// Mark chunk as reusable, inserting it as un-touched if not present
-    fn mark_reusable(&mut self, digest: &[u8; 32]) {
+    /// Mark chunk as reusable, inserting it as un-touched if not present.
+    ///
+    /// If the mapping already contains the digest, set the decrypted digest only
+    /// if not already set previously.
+    fn mark_reusable(&mut self, digest: &[u8; 32], decrypted_digest: Option<[u8; 32]>) {
         match self.chunk_set.entry(*digest) {
             Entry::Occupied(mut occupied) => {
-                let (reusable, _touched) = occupied.get_mut();
-                *reusable = true;
+                let chunk_info = occupied.get_mut();
+                chunk_info.reusable = true;
+                if chunk_info.decrypted_digest.is_none() {
+                    chunk_info.decrypted_digest = decrypted_digest;
+                }
             }
             Entry::Vacant(vacant) => {
-                vacant.insert((true, false));
+                vacant.insert(EncounteredChunkInfo {
+                    reusable: true,
+                    touched: false,
+                    decrypted_digest,
+                });
             }
         }
     }
 
     /// Mark chunk as touched during this sync, inserting it as not reusable
     /// but touched if not present.
-    fn mark_touched(&mut self, digest: &[u8; 32]) {
+    ///
+    /// If the mapping already contains the digest, set the decrypted digest only
+    /// if not already set previously.
+    fn mark_touched(&mut self, digest: &[u8; 32], decrypted_digest: Option<[u8; 32]>) {
         match self.chunk_set.entry(*digest) {
             Entry::Occupied(mut occupied) => {
-                let (_reusable, touched) = occupied.get_mut();
-                *touched = true;
+                let chunk_info = occupied.get_mut();
+                chunk_info.touched = true;
+                if chunk_info.decrypted_digest.is_none() {
+                    chunk_info.decrypted_digest = decrypted_digest;
+                }
             }
             Entry::Vacant(vacant) => {
-                vacant.insert((false, true));
+                vacant.insert(EncounteredChunkInfo {
+                    reusable: false,
+                    touched: true,
+                    decrypted_digest,
+                });
             }
         }
     }
-- 
2.47.3





  parent reply	other threads:[~2026-04-20 16:22 UTC|newest]

Thread overview: 31+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2026-04-20 16:15 [PATCH proxmox{,-backup} v4 00/30] fix #7251: implement server side encryption support for push sync jobs Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox v4 01/30] pbs-api-types: define en-/decryption key type and schema Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox v4 02/30] pbs-api-types: sync job: add optional cryptographic keys to config Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 03/30] sync: push: use tracing macros instead of log Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 04/30] datastore: blob: implement async reader for data blobs Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 05/30] datastore: manifest: add helper for change detection fingerprint Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 06/30] pbs-key-config: introduce store_with() for KeyConfig Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 07/30] pbs-config: implement encryption key config handling Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 08/30] pbs-config: acls: add 'encryption-keys' as valid 'system' subpath Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 09/30] ui: expose 'encryption-keys' as acl subpath for 'system' Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 10/30] sync: add helper to check encryption key acls and load key Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 11/30] api: config: add endpoints for encryption key manipulation Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 12/30] api: config: check sync owner has access to en-/decryption keys Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 13/30] api: config: allow encryption key manipulation for sync job Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 14/30] sync: push: rewrite manifest instead of pushing pre-existing one Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 15/30] api: push sync: expose optional encryption key for push sync Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 16/30] sync: push: optionally encrypt data blob on upload Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 17/30] sync: push: optionally encrypt client log on upload if key is given Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 18/30] sync: push: add helper for loading known chunks from previous snapshot Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 19/30] fix #7251: api: push: encrypt snapshots using configured encryption key Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 20/30] ui: define and expose encryption key management menu item and windows Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 21/30] ui: expose assigning encryption key to sync jobs Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 22/30] sync: pull: load encryption key if given in job config Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 23/30] sync: expand source chunk reader trait by crypt config Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 24/30] sync: pull: introduce and use decrypt index writer if " Christian Ebner
2026-04-20 16:15 ` Christian Ebner [this message]
2026-04-20 16:15 ` [PATCH proxmox-backup v4 26/30] sync: pull: decrypt blob files on pull if encryption key is configured Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 27/30] sync: pull: decrypt chunks and rewrite index file for matching key Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 28/30] sync: pull: decrypt snapshots with matching encryption key fingerprint Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 29/30] api: encryption keys: allow to toggle the archived state for keys Christian Ebner
2026-04-20 16:15 ` [PATCH proxmox-backup v4 30/30] docs: add section describing server side encryption for sync jobs Christian Ebner

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20260420161533.1055484-26-c.ebner@proxmox.com \
    --to=c.ebner@proxmox.com \
    --cc=pbs-devel@lists.proxmox.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
Service provided by Proxmox Server Solutions GmbH | Privacy | Legal